import timeit

import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np

from newton_type import newton_type


# for people who cannot see the plots
if mpl.get_backend() == 'agg':
    mpl.use('WebAgg')
print(f"backend: {mpl.get_backend()}")


## plot the objective function

# define the function as anonymous function handle
# you can then evaluate it as f(x,y) (elementwise)
f = lambda x,y: .5*((x-1)**2 + 100*(y-x**2)**2 + y**2)

# Create circle grid of x y values
dR = .002
N = int(np.sqrt(2) / dR)
R = np.linspace(0, np.sqrt(2), N)
TH = 2*np.pi*np.linspace(0, 1, 500) 
X = np.outer(R.T, np.cos(TH)) 
Y = np.outer(R.T, np.sin(TH))
# calculate objective function values
# add 1 and take log for better visualization
Z = np.log(1 + f(X,Y))

fig = plt.figure(1)


# subplot 1 with surface plot
ax = fig.add_subplot(1,2,1, projection='3d')
ax.view_init(100, 50)
ax.grid(True)
# ax.axis('square')  # NOTE: cannot scale Axes3D
ax.plot_surface(X,Y,Z, linewidth=0, cmap='coolwarm')
ax.set_xlabel(r'$x$')
ax.set_ylabel(r'$y$')
ax.set_zlabel(r'$\log(1+f(x,y))$')
# read current axis limits and fix them at these values
ax_lim = ax.axis()
ax.axis(ax_lim)

# subplot 2 with contour plot
ax = fig.add_subplot(1,2,2, projection='3d')
ax.view_init(100, 50)
ax.grid(True)
# ax.axis('square')  # NOTE: cannot scale Axes3D
ax.contour(X,Y,Z, cmap='gray')
ax.set_xlabel(r'$x$')
ax.set_ylabel(r'$y$')
ax.set_zlabel(r'$\log(1+f(x,y))$')
# fix the axis limits at the same values
ax.axis(ax_lim)

fig.suptitle(r'Surface and contour plot of the nonlinear function')


## grad and hessians

# gradient
fgrad = lambda x,y: [
    ...,
    ...
]

# true hessian
fhess = lambda x,y: [
    [..., ...],
    [..., ...]
]


## Implementation

# TODO: add your implementation of the Newton Type methods



## results
# you can add the plot of the iteration trajectory (xk,yk) here
fig.add_subplot(ax)




## show plots
plt.show()
